# Drizzle ORM 插件

这是一个集成到 Elysia 框架的 Drizzle ORM 插件，提供类型安全的数据库操作。

## 安装依赖

```bash
# 安装 Drizzle ORM 核心包
bun add drizzle-orm

# 安装 Drizzle Kit (开发工具)
bun add drizzle-kit -D
```

## 文件结构

```
src/plugins/drizzle/
├── drizzle.plugins.ts    # 主插件文件
├── drizzle.config.ts     # Drizzle Kit 配置
├── schema/               # 数据库表结构定义
│   ├── index.ts         # Schema 总入口
│   └── users.ts         # 用户表示例
└── README.md            # 使用说明
```

## 使用方法

### 1. 在路由中使用数据库

```typescript
import { Elysia } from 'elysia';
import { users } from '@/plugins/drizzle/schema';

const app = new Elysia()
    .get('/users', async ({ db }) => {
        // 查询所有用户
        const allUsers = await db.select().from(users);
        return allUsers;
    })
    .get('/users/:id', async ({ db, params }) => {
        // 根据ID查询用户
        const user = await db.select()
            .from(users)
            .where(eq(users.id, parseInt(params.id)));
        return user[0];
    })
    .post('/users', async ({ db, body }) => {
        // 创建新用户
        const newUser = await db.insert(users).values(body);
        return newUser;
    });
```

### 2. 定义新的表结构

在 `schema/` 目录下创建新的表文件：

```typescript
// schema/posts.ts
import { mysqlTable, int, varchar, text, timestamp } from 'drizzle-orm/mysql-core';
import { users } from './users';

export const posts = mysqlTable('posts', {
    id: int('id').primaryKey().autoincrement(),
    title: varchar('title', { length: 255 }).notNull(),
    content: text('content'),
    authorId: int('author_id').references(() => users.id),
    createdAt: timestamp('created_at').defaultNow().notNull(),
    updatedAt: timestamp('updated_at').defaultNow().onUpdateNow().notNull(),
});

export type Post = typeof posts.$inferSelect;
export type InsertPost = typeof posts.$inferInsert;
```

然后在 `schema/index.ts` 中导出：

```typescript
export * from './posts';
```

### 3. 生成和运行数据库迁移

```bash
# 生成迁移文件
bun drizzle-kit generate

# 推送迁移到数据库
bun drizzle-kit push

# 查看数据库状态
bun drizzle-kit studio
```

## 配置说明

- **数据库连接**: 自动从 `@/config` 读取数据库配置
- **连接池**: 默认最大连接数为 10
- **日志**: 启用 SQL 查询日志
- **迁移**: 迁移文件输出到 `./drizzle` 目录

## 类型支持

插件提供完整的 TypeScript 类型支持：

- `DrizzleDB`: 数据库实例类型
- `User`, `InsertUser`: 用户表相关类型
- `DrizzleContext`: Elysia 上下文扩展类型

## 注意事项

1. 确保数据库配置正确且数据库服务已启动
2. 生产环境建议使用环境变量管理数据库凭据
3. 定期备份数据库，特别是在运行迁移之前
4. 使用 Drizzle Studio 可视化管理数据库

## 相关链接

- [Drizzle ORM 官方文档](https://orm.drizzle.team/)
- [Drizzle Kit 文档](https://orm.drizzle.team/kit-docs/overview) 