/**
 * @file        邮件服务测试
 * @author      hotok
 * @date        2025-06-29
 * @lastEditor  hotok
 * @lastEditTime 2025-06-29
 * @description 测试邮件服务的连接、发送、模板、健康检查等功能
 */

import { describe, it, expect, beforeAll, afterAll, beforeEach } from 'vitest';
import { setTimeout } from 'node:timers';
import { 
    emailService, 
    initializeEmailService,
    sendEmail,
    sendTemplateEmail,
    getEmailServiceStatus,
    checkEmailServiceHealth,
    closeEmailService
} from '@/plugins/email/email.service';
import { validateEmailConfig } from '@/config/email.config';
import type { EmailSendOptions, EmailTemplateSendOptions } from '@/type/email.type';

// 简单的延时函数
const delay = (ms: number) => new Promise(resolve => setTimeout(resolve, ms));

// 测试用的邮箱地址（请根据实际情况修改）
const TEST_EMAIL = 'test@example.com';
const TEST_USERNAME = 'test_user';
const TEST_NICKNAME = '测试用户';

describe('邮件服务测试', () => {
    let isEmailConfigValid = false;

    beforeAll(async () => {
        // 检查邮件配置是否有效
        isEmailConfigValid = validateEmailConfig();
        
        if (isEmailConfigValid) {
            try {
                // 初始化邮件服务
                await initializeEmailService();
                console.log('邮件服务初始化成功');
            } catch (error) {
                console.warn('邮件服务初始化失败，某些测试将被跳过:', error);
                isEmailConfigValid = false;
            }
        } else {
            console.warn('邮件配置无效，邮件发送测试将被跳过');
        }
    });

    afterAll(async () => {
        if (isEmailConfigValid) {
            await closeEmailService();
        }
    });

    describe('邮件服务配置和初始化', () => {
        it('应该验证邮件配置有效性', () => {
            const isValid = validateEmailConfig();
            // 不强制要求配置有效，因为测试环境可能没有配置SMTP
            expect(typeof isValid).toBe('boolean');
        });

        it('应该能够获取邮件服务状态', () => {
            const status = getEmailServiceStatus();
            expect(status).toHaveProperty('status');
            expect(status).toHaveProperty('transporterStatus');
            expect(status).toHaveProperty('lastCheckAt');
            expect(['healthy', 'unhealthy', 'degraded']).toContain(status.status);
            expect(['connected', 'disconnected', 'error']).toContain(status.transporterStatus);
        });

        it('应该能够检查服务初始化状态', () => {
            const isInitialized = emailService.isInitialized;
            expect(typeof isInitialized).toBe('boolean');
        });
    });

    describe('邮件服务健康检查', () => {
        it('应该执行健康检查并返回结果', async () => {
            const healthResult = await checkEmailServiceHealth();
            
            expect(healthResult).toHaveProperty('status');
            expect(healthResult).toHaveProperty('responseTime');
            expect(healthResult).toHaveProperty('serviceStatus');
            expect(['healthy', 'unhealthy']).toContain(healthResult.status);
            expect(typeof healthResult.responseTime).toBe('number');
            expect(healthResult.responseTime).toBeGreaterThanOrEqual(0);
        });
    });

    describe('邮件发送功能', () => {
        // 只在邮件配置有效时运行实际发送测试
        const runSendTest = isEmailConfigValid ? it : it.skip;

        runSendTest('应该能够发送简单文本邮件', async () => {
            const emailOptions: EmailSendOptions = {
                to: TEST_EMAIL,
                subject: '测试邮件 - 简单文本',
                text: '这是一封测试邮件的纯文本内容。',
                html: '<p>这是一封测试邮件的<strong>HTML内容</strong>。</p>',
            };

            const result = await sendEmail(emailOptions);
            
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            expect(result.sentAt).toBeInstanceOf(Date);
            
            if (result.success) {
                expect(result).toHaveProperty('messageId');
                expect(result.accepted).toContain(TEST_EMAIL);
                console.log('邮件发送成功:', result.messageId);
            } else {
                console.warn('邮件发送失败:', result.error);
            }
        }, 10000); // 设置较长的超时时间

        runSendTest('应该能够发送带抄送的邮件', async () => {
            const emailOptions: EmailSendOptions = {
                to: TEST_EMAIL,
                cc: TEST_EMAIL, // 测试环境下抄送给同一个邮箱
                subject: '测试邮件 - 带抄送',
                html: '<h2>测试抄送功能</h2><p>这封邮件测试抄送功能。</p>',
            };

            const result = await sendEmail(emailOptions);
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
        }, 10000);

        it('应该处理邮件发送错误', async () => {
            const invalidEmailOptions: EmailSendOptions = {
                to: 'invalid-email', // 无效邮箱地址
                subject: '测试邮件 - 无效地址',
                text: '这应该失败',
            };

            const result = await sendEmail(invalidEmailOptions);
            
            // 无论SMTP是否配置，无效邮箱都应该返回失败
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            
            if (!result.success) {
                expect(result).toHaveProperty('error');
                expect(result.rejected).toContain('invalid-email');
            }
        });
    });

    describe('模板邮件功能', () => {
        const runTemplateTest = isEmailConfigValid ? it : it.skip;

        runTemplateTest('应该能够发送用户激活邮件', async () => {
            const templateOptions: EmailTemplateSendOptions = {
                to: TEST_EMAIL,
                template: 'activation',
                params: {
                    username: TEST_USERNAME,
                    nickname: TEST_NICKNAME,
                    email: TEST_EMAIL,
                    activationUrl: 'https://example.com/activate?token=test123',
                    activationCode: '123456',
                    expireTime: '24小时',
                },
            };

            const result = await sendTemplateEmail(templateOptions);
            
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            
            if (result.success) {
                expect(result).toHaveProperty('messageId');
                console.log('激活邮件发送成功:', result.messageId);
            }
        }, 10000);

        runTemplateTest('应该能够发送密码重置邮件', async () => {
            const templateOptions: EmailTemplateSendOptions = {
                to: TEST_EMAIL,
                template: 'passwordReset',
                params: {
                    username: TEST_USERNAME,
                    nickname: TEST_NICKNAME,
                    email: TEST_EMAIL,
                    resetUrl: 'https://example.com/reset?token=test123',
                    resetCode: '654321',
                    expireTime: '30分钟',
                },
            };

            const result = await sendTemplateEmail(templateOptions);
            
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            
            if (result.success) {
                console.log('密码重置邮件发送成功:', result.messageId);
            }
        }, 10000);

        runTemplateTest('应该能够发送欢迎邮件', async () => {
            const templateOptions: EmailTemplateSendOptions = {
                to: TEST_EMAIL,
                template: 'welcome',
                params: {
                    username: TEST_USERNAME,
                    nickname: TEST_NICKNAME,
                    email: TEST_EMAIL,
                },
            };

            const result = await sendTemplateEmail(templateOptions);
            
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            
            if (result.success) {
                console.log('欢迎邮件发送成功:', result.messageId);
            }
        }, 10000);

        runTemplateTest('应该能够发送密码修改通知邮件', async () => {
            const templateOptions: EmailTemplateSendOptions = {
                to: TEST_EMAIL,
                template: 'passwordChanged',
                params: {
                    username: TEST_USERNAME,
                    nickname: TEST_NICKNAME,
                    email: TEST_EMAIL,
                },
            };

            const result = await sendTemplateEmail(templateOptions);
            
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            
            if (result.success) {
                console.log('密码修改通知邮件发送成功:', result.messageId);
            }
        }, 10000);

        runTemplateTest('应该能够发送系统通知邮件', async () => {
            const templateOptions: EmailTemplateSendOptions = {
                to: TEST_EMAIL,
                template: 'notification',
                params: {
                    username: TEST_USERNAME,
                    nickname: TEST_NICKNAME,
                    message: '这是一条重要的系统通知，请及时查看您的账户状态。',
                },
                subject: '重要系统通知',
            };

            const result = await sendTemplateEmail(templateOptions);
            
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            
            if (result.success) {
                console.log('系统通知邮件发送成功:', result.messageId);
            }
        }, 10000);

        it('应该处理无效模板类型', async () => {
            const invalidTemplateOptions = {
                to: TEST_EMAIL,
                template: 'invalid-template' as any,
                params: {
                    username: TEST_USERNAME,
                },
            };

            const result = await sendTemplateEmail(invalidTemplateOptions);
            
            expect(result.success).toBe(false);
            expect(result).toHaveProperty('error');
            expect(result.error).toContain('未找到邮件模板');
        });
    });

    describe('邮件服务错误处理', () => {
        it('应该能够处理SMTP连接错误', async () => {
            // 模拟SMTP连接失败的情况
            const emailOptions: EmailSendOptions = {
                to: TEST_EMAIL,
                subject: '测试连接错误',
                text: '这个测试用于验证错误处理',
            };

            const result = await sendEmail(emailOptions);
            
            // 如果SMTP未配置或连接失败，应该有适当的错误处理
            expect(result).toHaveProperty('success');
            expect(result).toHaveProperty('sentAt');
            
            if (!result.success) {
                expect(result).toHaveProperty('error');
                expect(typeof result.error).toBe('string');
            }
        });
    });

    describe('邮件服务性能测试', () => {
        const runPerformanceTest = isEmailConfigValid ? it : it.skip;

        runPerformanceTest('应该能够并发发送多封邮件', async () => {
            const emailPromises = [];
            const emailCount = 3; // 限制并发数量以避免SMTP限制

            for (let i = 0; i < emailCount; i++) {
                const emailOptions: EmailSendOptions = {
                    to: TEST_EMAIL,
                    subject: `并发测试邮件 ${i + 1}`,
                    text: `这是第 ${i + 1} 封并发测试邮件`,
                };
                
                emailPromises.push(sendEmail(emailOptions));
            }

            const results = await Promise.all(emailPromises);
            
            expect(results).toHaveLength(emailCount);
            results.forEach((result, index) => {
                expect(result).toHaveProperty('success');
                expect(result).toHaveProperty('sentAt');
                console.log(`邮件 ${index + 1} 发送结果:`, result.success ? '成功' : '失败');
            });
        }, 30000); // 更长的超时时间用于并发测试
    });

    describe('邮件模板渲染测试', () => {
        it('应该正确渲染激活邮件模板内容', async () => {
            const templateOptions: EmailTemplateSendOptions = {
                to: TEST_EMAIL,
                template: 'activation',
                params: {
                    username: 'test_user',
                    nickname: '测试用户',
                    activationUrl: 'https://example.com/activate?token=abc123',
                    activationCode: '123456',
                    expireTime: '24小时',
                },
            };

            // 我们可以通过发送结果来验证模板是否正确处理
            const result = await sendTemplateEmail(templateOptions);
            expect(result).toHaveProperty('success');
            
            // 模板渲染错误会导致发送失败
            if (!result.success && result.error) {
                expect(result.error).not.toContain('模板渲染失败');
            }
        });
    });
}); 