/**
 * @file        Swagger文档功能测试
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 测试Swagger API文档功能，验证文档可访问性和内容完整性
 */

import { describe, it, expect } from 'vitest';
import { setTimeout } from 'node:timers';
import { app } from '@/app';

describe('Swagger API文档测试', () => {
    describe('GET /docs', () => {
        it('应该可以访问Swagger文档页面', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs', {
                    method: 'GET',
                }),
            );
            
            expect(res.status).toBe(200);
            
            const contentType = res.headers.get('content-type');
            expect(contentType).toContain('text/html');
        });

        it('Swagger文档应该包含基本配置信息', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs', {
                    method: 'GET',
                }),
            );
            
            const html = await res.text();
            
            // 检查基本配置
            expect(html).toContain('Cursor Init API服务');
            expect(html).toContain('swagger-ui');
        });
    });

    describe('GET /docs/json', () => {
        it('应该返回OpenAPI JSON文档', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            expect(res.status).toBe(200);
            
            const contentType = res.headers.get('content-type');
            expect(contentType).toContain('application/json');
            
            const openApiDoc = await res.json();
            
            // 验证OpenAPI文档结构
            expect(openApiDoc).toHaveProperty('openapi');
            expect(openApiDoc).toHaveProperty('info');
            expect(openApiDoc).toHaveProperty('paths');
            expect(openApiDoc).toHaveProperty('components');
            
            // 验证基本信息
            expect(openApiDoc.info.title).toBe('Cursor Init API服务');
            expect(openApiDoc.info.version).toBe('1.0.0');
            expect(openApiDoc.info.description).toContain('Cursor Init API服务');
        });

        it('OpenAPI文档应该包含安全配置', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证安全配置
            expect(openApiDoc.components).toHaveProperty('securitySchemes');
            expect(openApiDoc.components.securitySchemes).toHaveProperty('bearerAuth');
            expect(openApiDoc.components.securitySchemes.bearerAuth.type).toBe('http');
            expect(openApiDoc.components.securitySchemes.bearerAuth.scheme).toBe('bearer');
            expect(openApiDoc.components.securitySchemes.bearerAuth.bearerFormat).toBe('JWT');
        });

        it('OpenAPI文档应该包含全局组件定义', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证全局组件
            expect(openApiDoc.components).toHaveProperty('schemas');
            expect(openApiDoc.components).toHaveProperty('responses');
            
            // 验证响应组件
            const responses = openApiDoc.components.responses;
            expect(responses).toHaveProperty('Success');
            expect(responses).toHaveProperty('BadRequest');
            expect(responses).toHaveProperty('Unauthorized');
            expect(responses).toHaveProperty('Forbidden');
            expect(responses).toHaveProperty('NotFound');
            expect(responses).toHaveProperty('ValidationError');
            expect(responses).toHaveProperty('InternalError');
            expect(responses).toHaveProperty('ServiceUnavailable');
            
            // 验证Schema组件
            const schemas = openApiDoc.components.schemas;
            expect(schemas).toHaveProperty('BaseResponse');
            expect(schemas).toHaveProperty('SuccessResponse');
            expect(schemas).toHaveProperty('ErrorResponse');
            expect(schemas).toHaveProperty('PaginationResponse');
        });

        it('OpenAPI文档应该包含健康检查接口', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证健康检查接口
            expect(openApiDoc.paths).toHaveProperty('/api/health');
            expect(openApiDoc.paths).toHaveProperty('/api/health/detailed');
            
            const healthPath = openApiDoc.paths['/api/health'];
            expect(healthPath).toHaveProperty('get');
            expect(healthPath.get).toHaveProperty('tags');
            expect(healthPath.get.tags).toContain('健康检查');
            expect(healthPath.get).toHaveProperty('summary');
            expect(healthPath.get).toHaveProperty('description');
        });

        it('OpenAPI文档应该包含认证接口', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证认证接口
            expect(openApiDoc.paths).toHaveProperty('/api/auth/login');
            
            const loginPath = openApiDoc.paths['/api/auth/login'];
            expect(loginPath).toHaveProperty('post');
            expect(loginPath.post).toHaveProperty('tags');
            expect(loginPath.post).toHaveProperty('requestBody');
            expect(loginPath.post).toHaveProperty('responses');
        });

        it('OpenAPI文档应该包含标签分类', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证标签
            expect(openApiDoc).toHaveProperty('tags');
            expect(Array.isArray(openApiDoc.tags)).toBe(true);
            
            const tagNames = openApiDoc.tags.map((tag: any) => tag.name);
            expect(tagNames).toContain('认证管理');
            expect(tagNames).toContain('健康检查');
            
            // 验证标签描述
            const healthTag = openApiDoc.tags.find((tag: any) => tag.name === '健康检查');
            expect(healthTag).toHaveProperty('description');
            expect(healthTag.description).toContain('系统健康状态');
        });
    });

    describe('Swagger文档内容验证', () => {
        it('应该包含错误码说明', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证错误码描述在文档中
            expect(openApiDoc.info.description).toContain('错误码说明');
            expect(openApiDoc.info.description).toContain('| 错误码 | 说明 | 示例场景 |');
            expect(openApiDoc.info.description).toContain('| 0 | 操作成功 |');
            expect(openApiDoc.info.description).toContain('| 400 | 业务逻辑错误 |');
            expect(openApiDoc.info.description).toContain('| 401 | 身份认证失败 |');
        });

        it('应该包含认证说明', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证认证说明
            expect(openApiDoc.info.description).toContain('认证说明');
            expect(openApiDoc.info.description).toContain('Authorization: Bearer');
            expect(openApiDoc.components.securitySchemes.bearerAuth.description).toContain('JWT认证说明');
            expect(openApiDoc.components.securitySchemes.bearerAuth.description).toContain('Token有效期为24小时');
        });

        it('应该包含响应格式说明', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证响应格式说明
            expect(openApiDoc.info.description).toContain('响应格式');
            expect(openApiDoc.info.description).toContain('"code": 0');
            expect(openApiDoc.info.description).toContain('"message": "操作成功"');
            expect(openApiDoc.info.description).toContain('"data"');
        });

        it('应该包含示例响应', async () => {
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const openApiDoc = await res.json();
            
            // 验证示例响应
            const successResponse = openApiDoc.components.responses.Success;
            expect(successResponse.content['application/json']).toHaveProperty('examples');
            
            const errorResponse = openApiDoc.components.responses.BadRequest;
            expect(errorResponse.content['application/json']).toHaveProperty('examples');
            
            const unauthorizedResponse = openApiDoc.components.responses.Unauthorized;
            expect(unauthorizedResponse.content['application/json']).toHaveProperty('examples');
            expect(unauthorizedResponse.content['application/json'].examples).toHaveProperty('tokenExpired');
            expect(unauthorizedResponse.content['application/json'].examples).toHaveProperty('tokenInvalid');
            expect(unauthorizedResponse.content['application/json'].examples).toHaveProperty('notLoggedIn');
        });
    });

    describe('Swagger文档性能测试', () => {
        it('文档页面应该快速加载', async () => {
            const startTime = Date.now();
            
            const res = await app.fetch(
                new Request('http://localhost/docs', {
                    method: 'GET',
                }),
            );
            
            const endTime = Date.now();
            const responseTime = endTime - startTime;
            
            expect(res.status).toBe(200);
            expect(responseTime).toBeLessThan(1000); // 应该在1秒内完成
        });

        it('JSON文档应该快速响应', async () => {
            const startTime = Date.now();
            
            const res = await app.fetch(
                new Request('http://localhost/docs/json', {
                    method: 'GET',
                }),
            );
            
            const endTime = Date.now();
            const responseTime = endTime - startTime;
            
            expect(res.status).toBe(200);
            expect(responseTime).toBeLessThan(500); // JSON文档应该更快
        });
    });
}); 