/**
 * @file        随机Chalk背景颜色工具
 * @author      hotok
 * @date        2024-01-15
 * @lastEditor  hotok
 * @lastEditTime 2024-01-15
 * @description 提供随机生成chalk背景颜色的工具函数，支持基本颜色和明亮颜色
 */

import chalk from 'chalk';

/**
 * 基本背景颜色数组
 * @description 包含所有基本的chalk背景颜色方法
 */
const basicBackgroundColors = [
    chalk.bgBlack,
    chalk.bgRed,
    chalk.bgGreen,
    chalk.bgYellow,
    chalk.bgBlue,
    chalk.bgMagenta,
    chalk.bgCyan,
    chalk.bgWhite,
    chalk.bgGray,
] as const;

/**
 * 明亮背景颜色数组
 * @description 包含所有明亮的chalk背景颜色方法
 */
const brightBackgroundColors = [
    chalk.bgBlackBright,
    chalk.bgRedBright,
    chalk.bgGreenBright,
    chalk.bgYellowBright,
    chalk.bgBlueBright,
    chalk.bgMagentaBright,
    chalk.bgCyanBright,
    chalk.bgWhiteBright,
] as const;

/**
 * 所有背景颜色数组
 * @description 基本颜色和明亮颜色的合并数组
 */
const allBackgroundColors = [...basicBackgroundColors, ...brightBackgroundColors] as const;

/**
 * 颜色类型枚举
 * @description 定义可选的颜色类型
 */
export type ColorType = 'basic' | 'bright' | 'all';

/**
 * 获取随机索引
 * @param length 数组长度
 * @returns 随机索引
 * @example
 * getRandomIndex(5) // 返回0-4之间的随机数
 */
function getRandomIndex(length: number): number {
    return Math.floor(Math.random() * length);
}

/**
 * 生成随机chalk背景颜色函数
 * @param type 颜色类型：'basic' | 'bright' | 'all'
 * @returns chalk背景颜色函数
 * @example
 * const randomBg = getRandomBackgroundColor();
 * console.log(randomBg('Hello World'));
 *
 * const randomBasicBg = getRandomBackgroundColor('basic');
 * console.log(randomBasicBg('Basic Color'));
 *
 * const randomBrightBg = getRandomBackgroundColor('bright');
 * console.log(randomBrightBg('Bright Color'));
 */
export function getRandomBackgroundColor(type: ColorType = 'all'): typeof chalk.bgRed {
    let colorArray: readonly (typeof chalk.bgRed)[];

    switch (type) {
        case 'basic':
            colorArray = basicBackgroundColors;
            break;
        case 'bright':
            colorArray = brightBackgroundColors;
            break;
        case 'all':
        default:
            colorArray = allBackgroundColors;
            break;
    }

    /** 获取随机索引 */
    const randomIndex = getRandomIndex(colorArray.length);
    /** 返回随机选择的背景颜色函数 */
    return colorArray[randomIndex]!;
}

/**
 * 直接生成带随机背景颜色的文本
 * @param text 要着色的文本
 * @param type 颜色类型：'basic' | 'bright' | 'all'
 * @returns 带背景颜色的文本
 * @example
 * console.log(randomBackgroundText('Hello World'));
 * console.log(randomBackgroundText('Basic Color', 'basic'));
 * console.log(randomBackgroundText('Bright Color', 'bright'));
 */
export function randomBackgroundText(text: string, type: ColorType = 'all'): string {
    /** 获取随机背景颜色函数 */
    const randomBgColor = getRandomBackgroundColor(type);
    /** 返回着色后的文本 */
    return randomBgColor(text);
}

/**
 * 生成带随机背景颜色和白色文字的文本
 * @param text 要着色的文本
 * @param type 颜色类型：'basic' | 'bright' | 'all'
 * @returns 带背景颜色和白色文字的文本
 * @example
 * console.log(randomBackgroundWhiteText('Hello World'));
 * console.log(randomBackgroundWhiteText('Important Message', 'bright'));
 */
export function randomBackgroundWhiteText(text: string, type: ColorType = 'all'): string {
    /** 获取随机背景颜色函数 */
    const randomBgColor = getRandomBackgroundColor(type);
    /** 返回带白色文字和随机背景的文本 */
    return randomBgColor.white(text);
}

/**
 * 生成带随机背景颜色和黑色文字的文本
 * @param text 要着色的文本
 * @param type 颜色类型：'basic' | 'bright' | 'all'
 * @returns 带背景颜色和黑色文字的文本
 * @example
 * console.log(randomBackgroundBlackText('Hello World'));
 * console.log(randomBackgroundBlackText('Important Message', 'basic'));
 */
export function randomBackgroundBlackText(text: string, type: ColorType = 'all'): string {
    /** 获取随机背景颜色函数 */
    const randomBgColor = getRandomBackgroundColor(type);
    /** 返回带黑色文字和随机背景的文本 */
    return randomBgColor.black(text);
}

/**
 * 生成彩虹背景效果（每个字符不同背景色）
 * @param text 要着色的文本
 * @param type 颜色类型：'basic' | 'bright' | 'all'
 * @returns 彩虹背景效果的文本
 * @example
 * console.log(rainbowBackgroundText('Rainbow!'));
 * console.log(rainbowBackgroundText('Colorful', 'bright'));
 */
export function rainbowBackgroundText(text: string, type: ColorType = 'all'): string {
    return text
        .split('')
        .map((char) => {
            /** 为每个字符生成随机背景颜色 */
            const randomBgColor = getRandomBackgroundColor(type);
            return randomBgColor.white(char);
        })
        .join('');
}

/**
 * 预定义的背景颜色名称映射
 * @description 用于调试和展示的颜色名称
 */
export const colorNames = {
    basic: ['bgBlack', 'bgRed', 'bgGreen', 'bgYellow', 'bgBlue', 'bgMagenta', 'bgCyan', 'bgWhite', 'bgGray'],
    bright: [
        'bgBlackBright',
        'bgRedBright',
        'bgGreenBright',
        'bgYellowBright',
        'bgBlueBright',
        'bgMagentaBright',
        'bgCyanBright',
        'bgWhiteBright',
    ],
} as const;

/**
 * 展示所有可用的背景颜色
 * @param type 颜色类型：'basic' | 'bright' | 'all'
 * @example
 * showAllBackgroundColors(); // 展示所有颜色
 * showAllBackgroundColors('basic'); // 只展示基本颜色
 */
export function showAllBackgroundColors(type: ColorType = 'all'): void {
    console.log(chalk.bold.blue(`\n🎨 Chalk背景颜色展示 (${type}):\n`));

    if (type === 'basic' || type === 'all') {
        console.log(chalk.bold.yellow('基本背景颜色:'));
        basicBackgroundColors.forEach((colorFunc, index) => {
            const colorName = colorNames.basic[index];
            console.log(`  ${colorFunc.white(` ${colorName} `)} ${colorName}`);
        });
        console.log();
    }

    if (type === 'bright' || type === 'all') {
        console.log(chalk.bold.yellow('明亮背景颜色:'));
        brightBackgroundColors.forEach((colorFunc, index) => {
            const colorName = colorNames.bright[index];
            console.log(`  ${colorFunc.black(` ${colorName} `)} ${colorName}`);
        });
        console.log();
    }
}

export default {
    getRandomBackgroundColor,
    randomBackgroundText,
    randomBackgroundWhiteText,
    randomBackgroundBlackText,
    rainbowBackgroundText,
    showAllBackgroundColors,
    colorNames,
};
