/**
 * @file        Redis数据库连接工具
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 提供Redis连接实例、连接池和健康检查功能，供全局复用
 */

import { createClient, type RedisClientType } from 'redis';
import { redisConfig, getRedisUrl } from '@/config/redis.config';
import { Logger } from '@/plugins/logger/logger.service';

/**
 * Redis客户端实例
 */
class Redis {
    public redisClient: RedisClientType;
    /**
     * Redis连接状态
     */
    private isConnected = false;
    constructor() {
        this.redisClient = createClient({
            name: redisConfig.connectName,
            username: redisConfig.username,
            password: redisConfig.password,
            database: redisConfig.database,
            url: getRedisUrl(),
        });

        // 错误处理
        this.redisClient.on('error', (error) => {
            Logger.error(error as Error);
            this.isConnected = false;
        });

        this.redisClient.on('connect', () => {
            Logger.info('Redis客户端连接建立');
            this.isConnected = true;
        });

        this.redisClient.on('ready', () => {
            Logger.info('Redis客户端准备就绪');
        });

        this.redisClient.on('end', () => {
            Logger.info('Redis客户端连接结束');
            this.isConnected = false;
        });

        this.redisClient.on('reconnecting', () => {
            Logger.warn('Redis客户端正在重连');
        });

        // 初始化连接
        this.connectRedis().catch((error) => {
            Logger.error(error as Error);
        });
    }

    /**
     * 连接Redis
     * @returns Promise<void>
     */
    async connectRedis() {
        try {
            if (!this.isConnected) {
                await this.redisClient.connect();
                this.isConnected = true;
            }
        } catch (error) {
            Logger.error(error as Error);
            throw error;
        }
    }
    /**
     * 断开Redis连接
     * @returns Promise<void>
     */
    async disconnectRedis() {
        try {
            if (this.isConnected) {
                await this.redisClient.close();
                this.isConnected = false;
            }
        } catch (error) {
            Logger.error(error as Error);
            throw error;
        }
    }

    /**
     * Redis健康检查
     * @returns Promise<boolean> 返回连接状态
     */
    async checkRedisHealth() {
        try {
            await this.redisClient.ping();
            return true;
        } catch (error) {
            Logger.error(error as Error);
            return false;
        }
    }

    /**
     * 获取Redis连接状态
     * @returns 连接状态信息
     */
    getRedisStatus() {
        return {
            isConnected: this.isConnected,
            config: {
                host: redisConfig.host,
                port: redisConfig.port,
                database: redisConfig.database,
                connectName: redisConfig.connectName,
            },
        };
    }
}

/**
 * 导出Redis类，供创建新实例使用
 */
export { Redis };
